@@ -20,14 +20,12 @@
 
 */
 
+#include "tldevel.h"
 
-#include "global.h"
+#include <string.h>
+#define PARAMETERS_IMPORT
 #include "parameters.h"
 
-
-
-
-
 struct parameters*init_param(void)
 {
         struct parameters* param = NULL;
@@ -45,22 +43,23 @@ struct parameters*init_param(void)
         param->help_flag = 0;
         param->dump_internal = 0;
 
-        param->gpo = FLT_MAX;
-        param->gpe = FLT_MAX;
-        param->tgpe = FLT_MAX;
+        param->type = -1;
+
+        param->gpo = -1.0;
+        param->gpe = -1.0;
+        param->tgpe = -1.0;
         param->matadd = 0.0F;
         param->chaos = 0;
         param->nthreads = 4;
         param->clean = 0;
         param->unalign = 0;
+        param->quiet = 0;
         return param;
 ERROR:
         free_parameters(param);
         return NULL;
-
 }
 
-
 void free_parameters(struct parameters* param)
 {
         if(param){
@@ -70,3 +69,28 @@ void free_parameters(struct parameters* param)
                 MFREE(param);
         }
 }
+
+int check_msa_format_string(char* format)
+{
+        int ok = 0;
+        if(format){
+                if(strstr(format,"msf")){
+                        ok = 1;
+                }else if(strstr(format,"clu")){
+                        ok = 1;
+                }else if(strstr(format,"fasta")){
+                        ok = 1;
+                }else if(strstr(format,"fa")){
+                        ok = 1;
+                }else{
+                        ok = 0;
+                        ERROR_MSG("Format %s not recognized.",format);
+                }
+                if(!ok){
+                        ERROR_MSG("Format %s not recognized.",format);
+                }
+        }
+        return OK;
+ERROR:
+        return FAIL;
+}
