@@ -21,59 +21,33 @@
 */
 
 #include "tldevel.h"
-#include <stdio.h>
-#ifdef HAVE_OPENMP
-#include <omp.h>
-#endif
-
-#include "global.h"
-
 #include "tlmisc.h"
-
-#include "msa.h"
+#include "kalign/kalign.h"
+#include "version.h"
 #include "parameters.h"
-//#include "align_io.h"
-#include "alignment_parameters.h"
 
-#include "bisectingKmeans.h"
-/* #include "alignment.h" */
-#include "weave_alignment.h"
-
-#include "esl_stopwatch.h"
-#include "misc.h"
-#include <getopt.h>
+#include <stdio.h>
 #include <string.h>
-#include "idata.h"
-#include "alphabet.h"
-
-#include "aln_run.h"
-#include "aln_task.h"
+#include <unistd.h>
+#include <getopt.h>
 
 #define OPT_SET 1
-#define OPT_ALNPARAM 2
-#define OPT_RENAME 3
-#define OPT_REFORMAT 4
 #define OPT_SHOWW 5
 #define OPT_GPO 6
 #define OPT_GPE 7
 #define OPT_TGPE 8
-#define OPT_MATADD 9
 
-#define OPT_DEVTEST 10
-#define OPT_CHAOS 11
-#define OPT_DUMP_INTERNAL 12
-#define OPT_NTHREADS 14
+#define OPT_NTHREADS 10
 
-#define OPT_CLEAN 15
-#define OPT_UNALIGN 16
+#define OPT_ALN_TYPE 13
+
+static int set_aln_type(char* in, int* type );
 
 static int run_kalign(struct parameters* param);
-static int check_for_sequences(struct msa* msa);
 
 static int print_kalign_header(void);
 static int print_kalign_help(char * argv[]);
 static int print_kalign_warranty(void);
-static int print_AVX_warning(void);
 
 int print_kalign_help(char * argv[])
 {
@@ -87,10 +61,16 @@ int print_kalign_help(char * argv[])
         fprintf(stdout,"Options:\n\n");
 
         fprintf(stdout,"%*s%-*s: %s %s\n",3,"",MESSAGE_MARGIN-3,"--format","Output format." ,"[Fasta]"  );
-        fprintf(stdout,"%*s%-*s: %s %s\n",3,"",MESSAGE_MARGIN-3,"--reformat","Reformat existing alignment." ,"[NA]"  );
-        fprintf(stdout,"%*s%-*s: %s %s\n",3,"",MESSAGE_MARGIN-3,"--gpo","Gap open penalty." ,"[5.5]"  );
-        fprintf(stdout,"%*s%-*s: %s %s\n",3,"",MESSAGE_MARGIN-3,"--gpe","Gap extension penalty." ,"[2.0]"  );
-        fprintf(stdout,"%*s%-*s: %s %s\n",3,"",MESSAGE_MARGIN-3,"--tgpe","Terminal gap extension penalty." ,"[1.0]"  );
+
+        /* fprintf(stdout,"%*s%-*s: %s %s\n",3,"",MESSAGE_MARGIN-3,"--reformat","Reformat existing alignment." ,"[NA]"  ); */
+        fprintf(stdout,"%*s%-*s: %s %s\n",3,"",MESSAGE_MARGIN-3,"--type","Alignment type (rna, dna, internal)." ,"[rna]"  );
+        fprintf(stdout,"%*s%-*s  %s %s\n",3,"",MESSAGE_MARGIN-3,"","Options: protein, divergent (protein)" ,""  );
+        fprintf(stdout,"%*s%-*s  %s %s\n",3,"",MESSAGE_MARGIN-3,"","         rna, dna, internal (nuc)." ,""  );
+
+        fprintf(stdout,"%*s%-*s: %s %s\n",3,"",MESSAGE_MARGIN-3,"--gpo","Gap open penalty." ,"[]");
+        fprintf(stdout,"%*s%-*s: %s %s\n",3,"",MESSAGE_MARGIN-3,"--gpe","Gap extension penalty." ,"[]");
+        fprintf(stdout,"%*s%-*s: %s %s\n",3,"",MESSAGE_MARGIN-3,"--tgpe","Terminal gap extension penalty." ,"[]");
+        fprintf(stdout,"%*s%-*s: %s %s\n",3,"",MESSAGE_MARGIN-3,"-n/--nthreads","Number of threads." ,"[4]");
 
         fprintf(stdout,"%*s%-*s: %s %s\n",3,"",MESSAGE_MARGIN-3,"--version (-V/-v)","Prints version." ,"[NA]"  );
 
@@ -99,8 +79,6 @@ int print_kalign_help(char * argv[])
         fprintf(stdout,"Passing sequences via stdin:\n\n   cat input.fa | kalign -f fasta > out.afa\n\n");
         fprintf(stdout,"Combining multiple input files:\n\n   kalign seqsA.fa seqsB.fa seqsC.fa -f fasta > combined.afa\n\n");
 
-
-
         if(basename){
                 MFREE(basename);
         }
@@ -133,7 +111,7 @@ int print_kalign_warranty(void)
 int print_kalign_header(void)
 {
         fprintf(stdout,"\n");
-        fprintf(stdout,"Kalign (%s)\n", PACKAGE_VERSION);
+        fprintf(stdout,"Kalign (%s)\n", KALIGN_PACKAGE_VERSION);
         fprintf(stdout,"\n");
         fprintf(stdout,"Copyright (C) 2006,2019,2020,2021 Timo Lassmann\n");
         fprintf(stdout,"\n");
@@ -144,46 +122,22 @@ int print_kalign_header(void)
         fprintf(stdout,"\n");
         fprintf(stdout,"Please cite:\n");
 
-        /*        fprintf(stdout,"  Kalign 3: multiple sequence alignment of large data sets
-Timo Lassmann
-Bioinformatics, btz795, https://doi.org/10.1093/bioinformatics/btz795
-        */
         fprintf(stdout,"  Lassmann, Timo.\n");
         fprintf(stdout,"  \"Kalign 3: multiple sequence alignment of large data sets.\"\n");
         fprintf(stdout,"  Bioinformatics (2019) \n");
         fprintf(stdout,"  https://doi.org/10.1093/bioinformatics/btz795\n");
         fprintf(stdout,"\n");
-
-        /*fprintf(stdout,"  Lassmann, Timo, Oliver Frings, and Erik LL Sonnhammer.\n");
-        fprintf(stdout,"  \"Kalign2: high-performance multiple alignment of protein and\n");
-        fprintf(stdout,"  nucleotide sequences allowing external features.\"\n");
-        fprintf(stdout,"  Nucleic acids research 37.3 (2008): 858-865.\n");
-        fprintf(stdout,"\n");
-        fprintf(stdout,"  Lassmann, Timo, and Erik LL Sonnhammer. \"Kalign–an accurate and\n");
-        fprintf(stdout,"  fast multiple sequence alignment algorithm.\"\n  BMC bioinformatics 6.1 (2005): 298.\n");
-        fprintf(stdout,"\n");*/
-
         return OK;
 }
 
-int print_AVX_warning(void)
-{
-        fprintf(stdout,"\n");
-        fprintf(stdout,"WARNING: AVX2 instruction set not found!\n");
-        fprintf(stdout,"         Kalign will not run optimally.\n");
-        fprintf(stdout,"\n");
-        return OK;
-}
-
-
 int main(int argc, char *argv[])
 {
         int version = 0;
         int c;
         int showw = 0;
-        int devtest = 0;
         struct parameters* param = NULL;
         char* in = NULL;
+        char* in_type = NULL;
         RUNP(param = init_param());
 
         param->num_infiles = 0;
@@ -191,20 +145,12 @@ int main(int argc, char *argv[])
         while (1){
                 static struct option long_options[] ={
                         {"showw", 0,0,OPT_SHOWW },
-                        {"alnp", required_argument,0,OPT_ALNPARAM},
                         {"set", required_argument,0,OPT_SET},
                         {"format",  required_argument, 0, 'f'},
-                        {"reformat",  required_argument, 0, OPT_REFORMAT},
-                        {"devtest",  no_argument, 0, OPT_DEVTEST},
-                        {"dumpinternal",no_argument, 0, OPT_DUMP_INTERNAL},
-                        {"changename",  0, 0, OPT_RENAME},
-                        {"clean",  0, 0, OPT_CLEAN},
-                        {"unalign",  0, 0, OPT_UNALIGN},
+                        {"type",  required_argument, 0, OPT_ALN_TYPE},
                         {"gpo",  required_argument, 0, OPT_GPO},
                         {"gpe",  required_argument, 0, OPT_GPE},
                         {"tgpe",  required_argument, 0, OPT_TGPE},
-                        {"matadd",  required_argument, 0, OPT_MATADD},
-                        {"chaos",   required_argument, 0, OPT_CHAOS},
                         {"nthreads",  required_argument, 0, 'n'},
                         {"input",  required_argument, 0, 'i'},
                         {"infile",  required_argument, 0, 'i'},
@@ -227,42 +173,23 @@ int main(int argc, char *argv[])
                         break;
                 }
                 switch(c) {
-                case OPT_CLEAN:
-                        param->clean = 1;
-                        break;
-                case OPT_UNALIGN:
-                        param->unalign = 1;
-                        break;
-                case OPT_CHAOS:
-                        param->chaos = atoi(optarg);
-                        break;
-                case OPT_DUMP_INTERNAL:
-                        param->dump_internal = 1;
+                case OPT_ALN_TYPE:
+                        in_type = optarg;
                         break;
                 case OPT_SHOWW:
                         showw = 1;
                         break;
-                case OPT_ALNPARAM:
-                        param->aln_param_file = optarg;
-                        break;
                 case OPT_SET:
                         param->param_set = atoi(optarg);
                         break;
-                case OPT_RENAME:
-                        param->rename = 1;
-                        break;
                 case 'f':
                         param->format = optarg;
                         break;
                 case 'n':
                         param->nthreads = atoi(optarg);
                         break;
-                case OPT_REFORMAT:
-                        param->format = optarg;
-                        param->reformat = 1;
-                        break;
-                case OPT_DEVTEST:
-                        devtest =1 ;
+                case 'q':
+                        param->quiet = 1;
                         break;
                 case OPT_GPO:
                         param->gpo = atof(optarg);
@@ -270,12 +197,9 @@ int main(int argc, char *argv[])
                 case OPT_GPE:
                         param->gpe = atof(optarg);
                         break;
-                case OPT_TGPE :
+                case OPT_TGPE:
                         param->tgpe = atof(optarg);
                         break;
-                case OPT_MATADD :
-                        param->matadd = atof(optarg);
-                        break;
                 case 'h':
                         param->help_flag = 1;
                         break;
@@ -299,19 +223,17 @@ int main(int argc, char *argv[])
         }
 
         if(version){
-                fprintf(stdout,"%s %s\n",PACKAGE_NAME, PACKAGE_VERSION);
+                fprintf(stdout,"%s %s\n",KALIGN_PACKAGE_NAME, KALIGN_PACKAGE_VERSION);
                 free_parameters(param);
                 return EXIT_SUCCESS;
         }
 
         if(!param->dump_internal){
+                if(!param->quiet){
                 print_kalign_header();
+                }
         }
 
-#ifndef HAVE_AVX2
-        RUN(print_AVX_warning());
-#endif
-
         if(showw){
                 print_kalign_warranty();
                 free_parameters(param);
@@ -376,48 +298,8 @@ int main(int argc, char *argv[])
                 }
         }
 
-        /* Just for internal development & testing */
-        if(devtest){
-                char* datadir = NULL;
-                char* tmp = NULL;
-                int f;
-                f = 0;
-                for(c = 0; c < param->num_infiles;c++){
-                        param->infile[f] = param->infile[c];
-                        if(param->infile[c] != NULL){
-                                f++;
-                        }
-                }
-                param->num_infiles = f;
-                LOG_MSG("Start aln tests.");
-                datadir = getenv("testdatafiledir");
-                for(c = 0; c < param->num_infiles;c++){
-                        tmp = NULL;
-                        MMALLOC(tmp, sizeof(char) * 1024);
-                        if(!datadir){
-                                snprintf(tmp,1024, "%s",param->infile[c]);
-                        }else{
-                                snprintf(tmp,1024, "%s/%s", datadir, param->infile[c]);
-                        }
-                        param->infile[c] = tmp;
-                }
-        }
-
-        if(!param->format){
-                param->out_format = FORMAT_FA;
-        }else{
-                if(strstr(param->format,"msf")){
-                        param->out_format = FORMAT_MSF;
-                }else if(strstr(param->format,"clu")){
-                        param->out_format = FORMAT_CLU;
-                }else if(strstr(param->format,"fasta")){
-                        param->out_format  = FORMAT_FA;
-                }else if(strstr(param->format,"fa")){
-                        param->out_format  = FORMAT_FA;
-                }else{
-                        ERROR_MSG("Format %s not recognized.",param->format);
-                }
-        }
+        RUN(check_msa_format_string(param->format));
+        RUN(set_aln_type(in_type, &param->type));
 
         if(param->num_infiles == 0){
                 if (!isatty(fileno(stdin))){
@@ -432,22 +314,22 @@ int main(int argc, char *argv[])
                 }
         }
 
-        if(param->chaos){
-                if(param->chaos == 1){
-                        ERROR_MSG("Param chaos need to be bigger than 1 (currently %d)", param->chaos);
-                }
-                if(param->chaos > 10){
-                        ERROR_MSG("Param chaos bigger than 10 (currently %d)",param->chaos);
-                }
-        }
+        /* if(param->chaos){ */
+        /*         if(param->chaos == 1){ */
+        /*                 ERROR_MSG("Param chaos need to be bigger than 1 (currently %d)", param->chaos); */
+        /*         } */
+        /*         if(param->chaos > 10){ */
+        /*                 ERROR_MSG("Param chaos bigger than 10 (currently %d)",param->chaos); */
+        /*         } */
+        /* } */
 
         RUN(run_kalign(param));
 
-        if(devtest){
-                for(c = 0; c < param->num_infiles;c++){
-                        MFREE(param->infile[c]);
-                }
-        }
+        /* if(devtest){ */
+        /*         for(c = 0; c < param->num_infiles;c++){ */
+        /*                 MFREE(param->infile[c]); */
+        /*         } */
+        /* } */
 
         free_parameters(param);
         return EXIT_SUCCESS;
@@ -459,179 +341,56 @@ ERROR:
 int run_kalign(struct parameters* param)
 {
         struct msa* msa = NULL;
-        struct msa* tmp_msa = NULL;
-        struct aln_param* ap = NULL;
-        struct aln_tasks* tasks = NULL;
-
-        int i;
-
-#ifdef HAVE_OPENMP
-        omp_set_nested(1);
-        omp_set_num_threads(param->nthreads);
-#endif
 
         if(param->num_infiles == 1){
-                RUN(read_input(param->infile[0],&msa));
+                RUN(kalign_read_input(param->infile[0], &msa,param->quiet));
         }else{
-                for(i = 0; i < param->num_infiles;i++){
-                        RUN(read_input(param->infile[i],&tmp_msa));
-                        if(tmp_msa){
-                                RUN(merge_msa(&msa, tmp_msa));
-                                free_msa(tmp_msa);
-                                tmp_msa = NULL;
-                        }
+                for(int i = 0; i < param->num_infiles;i++){
+                        RUN(kalign_read_input(param->infile[i], &msa,param->quiet));
                 }
         }
-        /* check if we have sequences  */
-        RUN(check_for_sequences(msa));
 
-        /* extra checks for input alignments */
-        if(param->clean){
-                RUN(run_extra_checks_on_msa(msa));
-        }
 
-        LOG_MSG("Detected: %d sequences.", msa->numseq);
-        /* If we just want to reformat end here */
-        if(param->reformat){
-                //LOG_MSG("%s reformat",param->reformat);
-                for (i = 0 ;i < msa->numseq;i++){
-                        msa->nsip[i] = i;
-                }
 
-                if(param->rename){
-                        for (i = 0 ;i < msa->numseq;i++){
-                                snprintf(msa->sequences[i]->name, 128, "SEQ%d", i+1);
-                        }
-                }
+        RUN(kalign_run(msa,
+                       param->nthreads,
+                       param->type,
+                       param->gpo,
+                       param->gpe,
+                       param->tgpe));
 
-                if(param->unalign){
-                        /* if(param->out_format == FORMAT_FA){ */
-                        RUN(dealign_msa(msa));
-                }
-
-                if(param->out_format != FORMAT_FA && msa->aligned != ALN_STATUS_ALIGNED){
-                        ERROR_MSG("Input sequences are not aligned - cannot write to MSA format: %s", param->format);
-                }
-
-                RUN(write_msa(msa, param->outfile, param->out_format));
-
-                free_msa(msa);
-                return OK;
-        }
-
-        if(msa->aligned != ALN_STATUS_UNALIGNED){
-                RUN(dealign_msa(msa));
-        }
-
-        /* allocate aln parameters  */
-        RUN(init_ap(&ap,param,msa->L ));
-
-        if(param->dump_internal){
-                double* s;
-                int s_len;
-                RUN(get_internal_data(msa,ap,&s, &s_len));
-                for(i = 0; i < s_len;i++){
-                        fprintf(stdout,"%0.2f ", s[i]);
-                }
-                fprintf(stdout,"\n");
-                MFREE(s);
-                free_msa(msa);
-                return OK;
-        }
-
-        /* Allocate tasks  */
-        RUN(alloc_tasks(&tasks, msa->numseq));
-
-        /* Start bi-secting K-means sequence clustering */
-        if(!param->chaos){
-#ifdef HAVE_OPENMP
-                i = floor(log((double) param->nthreads) / log(2.0)) + 4;
-                i = MACRO_MIN(i, 10);
-                /* LOG_MSG("Set %d level (%d)", i, param->nthreads); */
-                omp_set_max_active_levels(i);
-#endif
-                RUN(build_tree_kmeans(msa,ap,&tasks));
-        }
-        /* by default all protein sequences are converted into a reduced alphabet
-           when read from file. Here we turn them back into the default representation. */
-        if(msa->L == ALPHA_redPROTEIN){
-                //RUN(convert_msa_to_internal(msa, defPROTEIN));
-                RUN(convert_msa_to_internal(msa, ALPHA_ambigiousPROTEIN));
-        }
-        /* allocate aln parameters  */
-        RUN(init_ap(&ap,param,msa->L ));
-
-        /* Start alignment stuff */
-        DECLARE_TIMER(t1);
-        LOG_MSG("Aligning");
-        START_TIMER(t1);
-
-        /* testing  */
-#ifdef HAVE_OPENMP
-        i = floor(log((double) param->nthreads) / log(2.0)) + 2;
-        i = MACRO_MIN(i, 10);
-        /* LOG_MSG("Set %d level (%d)", i, param->nthreads); */
-        omp_set_max_active_levels(i);
-#endif
-        if(param->chaos){
-
-                RUN(create_chaos_msa_openMP(msa, ap,tasks));
-
-        }else{
-                RUN(create_msa_tree(msa, ap, tasks,param->nthreads));
-        }
-        /* RUN(create_msa_openMP(msa,ap, tasks)); */
-
-/*         if(param->chaos){ */
-/* #ifdef HAVE_OPENMP */
-/*                 RUN(create_chaos_msa_openMP(msa, ap,tasks)); */
-/* #else */
-/*                 RUN(create_chaos_msa_serial(msa, ap,tasks)); */
-/* #endif */
-/*         }else{ */
-/* #ifdef HAVE_OPENMP */
-/*                 RUN(create_msa_openMP(msa,ap, tasks)); */
-/* #else */
-/*                 RUN(create_msa_serial(msa,ap, tasks)); */
-/* #endif */
-/*         } */
-        //RUNP(map = hirschberg_alignment(msa, ap));
-        STOP_TIMER(t1);
-        GET_TIMING(t1);
-
-        /* set to aligned */
-        msa->aligned = ALN_STATUS_ALIGNED;
-
-        /* clean up map */
-        free_tasks(tasks);
-
-        /* We are done. */
-        RUN(write_msa(msa, param->outfile, param->out_format));
-
-        free_msa(msa);
-        free_ap(ap);
-        DESTROY_TIMER(t1);
+        RUN(kalign_write_msa(msa, param->outfile, param->format));
+        kalign_free_msa(msa);
         return OK;
 ERROR:
-        free_msa(tmp_msa);
-        free_msa(msa);
+        kalign_free_msa(msa);
         return FAIL;
 }
 
-int check_for_sequences(struct msa* msa)
 
+int set_aln_type(char* in, int* type )
 {
-        if(!msa){
-                ERROR_MSG("No sequences were found in the input files or standard input.");
-        }
-        if(msa->numseq < 2){
-                if(msa->numseq == 0){
-                        ERROR_MSG("No sequences were found in the input files or standard input.");
-                }else if (msa->numseq == 1){
-                        ERROR_MSG("Only 1 sequence was found in the input files or standard input");
+        int t = 0;
+        if(in){
+                if(strstr(in,"rna")){
+                        t = KALIGN_TYPE_RNA;
+                }else if(strstr(in,"dna")){
+                        t = KALIGN_TYPE_DNA;
+                }else if(strstr(in,"internal")){
+                        t = KALIGN_TYPE_DNA_INTERNAL;
+                }else if(strstr(in,"protein")){
+                        t = KALIGN_TYPE_PROTEIN;
+                }else if(strstr(in,"divergent")){
+                        t = KALIGN_TYPE_PROTEIN_DIVERGENT;
+                }else{
+                        ERROR_MSG("In %s not recognized.",in);
                 }
+        }else{
+                t = KALIGN_TYPE_UNDEFINED;
         }
+        *type = t;
         return OK;
 ERROR:
         return FAIL;
 }
+
